
import os, webbrowser
import random, threading, logging

import google_auth_oauthlib.flow

import flask #for temporary redirection server.
from flask import current_app, Blueprint

from . import creds_helper

#blueprint, which we register with temporary server app in Authorizer factory class.
auth_bp = Blueprint('auth', __name__)

@auth_bp.route('/authorize')
def authorize():
    print('\033[H\033[J')
    print ('now complete authorisation in your web browser.')
    flow = google_auth_oauthlib.flow.Flow.from_client_secrets_file(current_app.config['client_secret_path'], scopes=current_app.config['scopes'])
    flow.redirect_uri = flask.url_for('auth.oauth2callback', _external=True)
    authorization_url, state = flow.authorization_url(
        access_type=current_app.config['access_type'],
        include_granted_scopes=current_app.config['include_granted_scopes']
        )
    return flask.redirect(authorization_url)


@auth_bp.route('/oauth2callback')
def oauth2callback():
    flow = google_auth_oauthlib.flow.Flow.from_client_secrets_file(current_app.config['client_secret_path'], scopes=current_app.config['scopes'])
    flow.redirect_uri = flask.url_for('auth.oauth2callback', _external=True)
    
    authorization_response = flask.request.url
    flow.fetch_token(authorization_response=authorization_response)
    credentials = flow.credentials
    
    credentials_file = open(current_app.config['credentials_storage_path'], 'wb')
    credentials_json = flask.json.dump(creds_helper.credentials_to_dict(credentials), credentials_file)

    print('\033[H\033[J')
    print ('authorization completed successfully, credentials stored in {}'.format(current_app.config['credentials_storage_path']))
    shutdown_server()
    return 'credentials successfully stored in {}'.format(current_app.config['credentials_storage_path'])

def shutdown_server():
    func = flask.request.environ.get('werkzeug.server.shutdown')
    if func is None:
        raise RuntimeError('Not running with the Werkzeug Server')
    func()




class Authorizer(object):

    def __init__(self, client_secret_path, credentials_storage_path, scopes, **kwargs):
        super(Authorizer, self).__init__()
        self.server_app = flask.Flask(__name__)
        self.server_app.secret_key = "somERandoMKey"

        self.server_app.config['client_secret_path'] = client_secret_path
        self.server_app.config['credentials_storage_path'] = credentials_storage_path
        self.server_app.config['scopes'] = scopes

        self.server_app.config['access_type'] = kwargs.get('access_type', 'offline')
        self.server_app.config['include_granted_scopes'] = kwargs.get('include_granted_scopes', 'true')

        self.server_app.register_blueprint(auth_bp)

    def authorize(self):
        log = logging.getLogger('werkzeug')
        log.setLevel(logging.ERROR)
        os.environ['OAUTHLIB_INSECURE_TRANSPORT'] = '1'
        port = 5000 + random.randint(0, 999)
        url = 'http://127.0.0.1:{port}/authorize'.format(port=port)
        threading.Timer(1.25, lambda: webbrowser.open(url)).start()
        self.server_app.run(port=port, debug=False)






from .authorization_helper import Authorizer

'''
usage is like fallows:

from oauth import Authorizer

authorizer = Authorizer(client_secret_path='path/to/client_secret.json', credentials_storage_path='path/to/credentials.json', scopes=['https://www.googleapis.com/auth/drive.readonly', 'https://www.googleapis.com/auth/spreadsheets.readonly'])

authorizer.authorize()
'''




import csv
import json, re
from functools import reduce #for 3.x if needed

import googleapiclient.discovery
from googleapiclient.errors import HttpError
from httplib2 import ServerNotFoundError
from google.auth.exceptions import TransportError

from ..oauth import creds_helper

SPREADSHEET_MIMETYPES = ['application/vnd.google-apps.spreadsheet']


def build_service(credentials):
    sheet_service = googleapiclient.discovery.build('sheets', 'v4', credentials=credentials, cache_discovery=False)
    return sheet_service

def build_service_from_creds_file(creds_file, scopes, auth_through_service_account=False):
    credentials = creds_helper.credentials_from_file(creds_file, scopes, auth_through_service_account)
    return build_service(credentials)

def build_service_from_creds_string(creds_string, scopes, auth_through_service_account=False):
    credentials = creds_helper.credentials_from_string(creds_string, scopes, auth_through_service_account)
    return build_service(credentials)

def error_response(**kwargs):
    #print ('in error_response')
    response = {'error' : {}}
    is_error_inherited = 'inherited_error_table' in kwargs
    response_code = kwargs.get('code', kwargs['inherited_error_table']['error'].get('code', 500) if is_error_inherited else 500)
    response_message = kwargs.get('message', None)

    response['error']['code'] = response_code
    if response_message is not None :
        response['error']['message'] = response_message
    if is_error_inherited :
        response['error']['inherited_error'] = kwargs['inherited_error_table']['error']

    return (response), response_code


class GSheets(object):

    def __init__(self, sheets_service, drive_service=None, **kwargs):
        if not isinstance(sheets_service, googleapiclient.discovery.Resource):
            raise TypeError('given service parameter is not googleapiclient.discovery.Resource object')

        self.sheets_service = sheets_service

        # drive-service is used for retrieving sheet's modification time, etc. which is not possible through sheets-api
        # drive-service is not mandatory. if it is not given, then only functionality related to it will be disabled.
        if isinstance(drive_service, googleapiclient.discovery.Resource):
            self.drive_service = drive_service
            self.enable_drive_service_linking = True
        else:
            self.drive_service = None
            self.enable_drive_service_linking = False

        self.EXPOSE_BACKEND_ERRORS = kwargs.get('expose_backend_errors', True)

    @classmethod
    def from_credentials(cls, credentials, **kwargs):
        service = build_service(credentials)
        return cls(service, drive_service=None, **kwargs)

    @classmethod
    def from_creds_file(cls, creds_file, scopes, auth_through_service_account=False, **kwargs):
        credentials = creds_helper.credentials_from_file(creds_file, scopes, auth_through_service_account=auth_through_service_account)
        return cls.from_credentials(credentials, **kwargs)

    @classmethod
    def from_creds_string(cls, creds_string, scopes, auth_through_service_account=False, **kwargs):
        credentials = creds_helper.credentials_from_string(creds_string, scopes, auth_through_service_account=auth_through_service_account)
        return cls.from_credentials(credentials, **kwargs)

    #TODO give helper factory method, which includes gdrive_functionality also.

    def spreadsheet_details_for(self, spreadsheet_id, filter_hashes_in_hdrs=True, hdr_split_char=' ', pargs=None):
        pargs = {} if pargs is None else pargs
        include_sheet_header_details = pargs.get('includeSheetHeaderDetails', 1)
        sheets_list = []
        try:
            sheets_response_table = self.sheets_service.spreadsheets().get(spreadsheetId=spreadsheet_id).execute()
        except HttpError as err:
            return error_response(message='error in getting details of requested spreadsheet',
                                  inherited_error_table=json.loads(err.content))
        except (ServerNotFoundError, TransportError) as err:
            return error_response(
                message='cannot connect to googleapis server to get spreadsheet details - ServerNotFoundError' if self.EXPOSE_BACKEND_ERRORS else 'cannot retrieve spreadsheet details')

        if self.enable_drive_service_linking:
            try:
                drive_meta_response_table = self.drive_service.files().get(fileId=spreadsheet_id, fields='id, name, mimeType, createdTime, modifiedTime').execute()
            except HttpError as err:
                return error_response(message='error in getting meta details of requested spreadsheet',
                                      inherited_error_table=json.loads(err.content))
            except (ServerNotFoundError, TransportError) as err:
                return error_response(
                    message='cannot connect to googleapis server to get spreadsheet meta information from drive - ServerNotFoundError' if self.EXPOSE_BACKEND_ERRORS else 'cannot retrieve meta details of spreadsheet')

        response_sheets_list = sheets_response_table.get('sheets', [])

        if include_sheet_header_details:
            try:
                sheets_headers_table = self.sheets_service.spreadsheets().values().batchGet(
                    spreadsheetId=spreadsheet_id,
                    ranges=[s['properties']['title'] + '!1:1' for s in response_sheets_list]
                ).execute()
            except HttpError as err:
                return error_response(message='error in getting sheet header fields details in requested spreadsheet',
                                      inherited_error_table=json.loads(err.content))
            except (ServerNotFoundError, TransportError) as err:
                return error_response(
                    message='cannot connect to googleapis server to get spreadsheet sheet header fields - ServerNotFoundError' if self.EXPOSE_BACKEND_ERRORS else 'cannot retrieve spreadsheet header fields')

        for count, sheet in enumerate(response_sheets_list):
            new_sheet = {}
            new_sheet['sheetGId'] = sheet.get('properties').get('sheetId')
            new_sheet['sheetTitle'] = sheet.get('properties').get('title')
            new_sheet['gridProperties'] = sheet.get('properties').get('gridProperties')

            if include_sheet_header_details:
                sheet_headers_table = sheets_headers_table['valueRanges'][count]
                headers_row = [s.lstrip() for s in sheet_headers_table['values'][0]]
                if filter_hashes_in_hdrs and headers_row[0][0] == '#':
                    headers_row[0] = headers_row[0][1:]
                hdr_split = [e.split(hdr_split_char, 1) if hdr_split_char in e else [e, ''] for e in headers_row]
                (cols, coldescs) = zip(*hdr_split)
                new_sheet['fields'] = list(cols)
                new_sheet['fieldDescs'] = list(coldescs)

            sheets_list.append(new_sheet)

        our_response_table = {'kind': 'spreadsheetDetails', 'spreadsheetId': spreadsheet_id}
        our_response_table['spreadsheetUrl'] = sheets_response_table.get('spreadsheetUrl')
        our_response_table['spreadsheetName'] = sheets_response_table.get('properties').get('title')

        if self.enable_drive_service_linking:
            #fallowing details are only details we get from drive-service
            our_response_table['createdTime'] = drive_meta_response_table.get('createdTime')
            our_response_table['modifiedTime'] = drive_meta_response_table.get('modifiedTime')

        our_response_table['sheets'] = sheets_list

        return (our_response_table), 200


    def sheet_values_for(self, spreadsheet_id, sheet_id, filter_hashes=True, hdr_split_char=' ', pargs=None):
        pargs = {} if pargs is None else pargs
        id_type = pargs.get('idType', 'gid')
        values_format = pargs.get('valuesFormat', 'maps')
        reqd_fields = pargs.get('fields', None)
        reqd_range_str = pargs.get('range', None)
        value_render_option = pargs.get('valueRenderOption', 'FORMATTED_VALUE')
        date_time_render_option = pargs.get('dateTimeRenderOption', 'SERIAL_NUMBER')

        if reqd_range_str:
            reqd_range_match = re.compile(r'^(\d+):(\d+)$').match(reqd_range_str)
            if reqd_range_match is None or int(reqd_range_match.group(1)) < 1:
                return error_response(code=400, message='illegal range')

        if id_type not in ['gid', 'title']:
            return error_response(code=400, message='invalid idType parameter')

        if values_format not in ['rows', 'columns', 'maps']:
            return error_response(code=400, message='invalid valuesFormat parameter')

        if value_render_option not in ['FORMATTED_VALUE', 'UNFORMATTED_VALUE', 'FORMULA']:
            return error_response(code=400, message='invalid valueRenderOption parameter')

        if date_time_render_option not in ['SERIAL_NUMBER', 'FORMATTED_STRING']:
            return error_response(code=400, message='invalid dateTimeRenderOption parameter')

        sheet_id = int(sheet_id) if id_type == 'gid' else sheet_id

        try:
            sheets_response_table = self.sheets_service.spreadsheets().get(spreadsheetId=spreadsheet_id).execute()
        except HttpError as err:
            return error_response(message='error in getting sheets details in requested spreadsheet',
                                  inherited_error_table=json.loads(err.content) if self.EXPOSE_BACKEND_ERRORS else {})
        except (ServerNotFoundError, TransportError) as err:
            return error_response(
                message='cannot connect to googleapis server to get sheets details in requested spreadsheet - ServerNotFoundError' if self.EXPOSE_BACKEND_ERRORS else 'cannot retrieve sheets details in requested spreadsheet')

        response_sheets_list = sheets_response_table.get('sheets', [])
        sheet_gid_title_map = {}
        for sheet in response_sheets_list:
            _sheet_gid = sheet.get('properties').get('sheetId')
            _sheet_title = sheet.get('properties').get('title')
            sheet_gid_title_map[_sheet_gid] = _sheet_title

        is_sheet_in_spreadsheet = (sheet_id in sheet_gid_title_map.keys()) if (id_type == 'gid') else (
                    sheet_id in sheet_gid_title_map.values())

        if not is_sheet_in_spreadsheet:
            return error_response(code=404, message='requested sheet is not there in parent spreadsheet')

        sheet_title = (sheet_id) if id_type == 'title' else sheet_gid_title_map.get(sheet_id)
        sheet_gid = sheet_id if id_type == 'gid' else reduce(
            (lambda x, y: x if (sheet_gid_title_map.get(x) == sheet_id) else y), sheet_gid_title_map.keys())

        reqd_range = (sheet_title + '!' + '{fr}:{tr}'.format(fr=int(reqd_range_match.group(1)) + 1, tr=int(
            reqd_range_match.group(2)) + 1)) if reqd_range_str else sheet_title + '!2:2000000'
        request_ranges = [sheet_title + '!1:1', reqd_range]

        try:
            response_table = self.sheets_service.spreadsheets().values().batchGet(
                spreadsheetId=spreadsheet_id,
                ranges=request_ranges,
                valueRenderOption=value_render_option,
                dateTimeRenderOption=date_time_render_option
            ).execute()
        except HttpError as err:
            return error_response(message='error in getting sheet values in requested spreadsheet',
                                  inherited_error_table=json.loads(err.content))
        except (ServerNotFoundError, TransportError) as err:
            return error_response(
                message='cannot connect to googleapis server to get values in requested spreadsheet - ServerNotFoundError' if self.EXPOSE_BACKEND_ERRORS else 'cannot retrieve sheet values in requested spreadsheet')

        table = {'sheetTitle': sheet_title}

        headers_range_values = response_table.get('valueRanges')[0].get('values')
        if not len(headers_range_values):
            return error_response(code=404, message='requested sheet is empty')
        headers_row = [s.lstrip() for s in headers_range_values[0]]
        if filter_hashes and headers_row[0][0] == '#':
            headers_row[0] = headers_row[0][1:]

        hdr_split = [e.split(hdr_split_char, 1) if hdr_split_char in e else [e, ''] for e in headers_row]
        (cols, coldescs) = zip(*hdr_split)
        idx = dict(zip(cols, range(len(cols))))
        table['fields'] = list(cols)
        table['descs'] = list(coldescs)
        table['values'] = []

        for row in response_table.get('valueRanges')[1].get('values'):
            if not len(row):
                continue

            if filter_hashes and row[0].startswith('#'):
                continue

            if (not True in [not (re.match(r'^[\s\-]*$', e)) for e in row]):
                # TODO provide option to give filler charcters to be filtered.
                # should check a way to escape regex components in a string
                continue

            values_row = row
            if len(values_row) < len(table.get('fields')):
                values_row += [''] * (len(table.get('fields')) - len(values_row))

            table['values'].append(values_row)

        reqd_fields = table.get('fields') if (reqd_fields is None) else reqd_fields
        fields_not_existed = [rf for rf in reqd_fields if rf not in table.get('fields')]

        if len(fields_not_existed):
            return error_response(code=404, message='fields ' + reduce((lambda x, y: x + ',' + y), fields_not_existed)[:] + ' doesn\'t exist in requested sheet.')

        reqd_field_indices = [idx[field] for field in reqd_fields]
        final_table = {'sheetTitle': sheet_title}
        final_table['fields'] = [table['fields'][i] for i in reqd_field_indices]
        final_table['descs'] = [table['descs'][i] for i in reqd_field_indices]
        final_table['values'] = [[table['values'][i][j] for j in reqd_field_indices] for i in
                                 range(len(table['values']))]

        our_response_table = {
            'spreadsheetId': spreadsheet_id,
            'sheetGId': sheet_gid,
            'sheetTitle': sheet_title,
            'valuesFormat': values_format,
            'fields': final_table['fields'],
            'fieldDescs': final_table['descs']
        }

        our_response_table['values'] = {
            'rows': final_table['values'],
            'columns': [[final_table['values'][i][j] for i in range(len(final_table['values']))] for j in
                        range(len(final_table['fields']))],
            'maps': [dict(zip(final_table['fields'], valrow)) for valrow in final_table['values']]
        }.get(values_format)

        return our_response_table, 200

    def raw_get(self, spreadsheet_id, pargs=None):
        pargs = {} if pargs is None else pargs

        try:
            sheets_response_table = self.sheets_service.spreadsheets().get(spreadsheetId=spreadsheet_id).execute()

        except HttpError as err:
            error_table = json.loads(err.content)
            return error_table, error_table.get('error', {}).get('code', 500)

        except (ServerNotFoundError, TransportError) as err:
            return error_response(
                message='cannot connect to googleapis server to get spreadsheet details - ServerNotFoundError' if self.EXPOSE_BACKEND_ERRORS else 'cannot retrieve spreadsheet details')

        return sheets_response_table, 200


    def raw_values_get(self, spreadsheet_id, range_string, pargs=None):
        pargs = {} if pargs is None else pargs

        try:
            response_table = self.sheets_service.spreadsheets().values().get(
                spreadsheetId=spreadsheet_id,
                range=range_string,
                majorDimension=pargs.get('majorDimension', 'ROWS'),
                valueRenderOption=pargs.get('valueRenderOption', 'FORMATTED_VALUE'),
                dateTimeRenderOption=pargs.get('dateTimeRenderOption', 'SERIAL_NUMBER')
            ).execute()

        except HttpError as err:
            error_table = json.loads(err.content)
            return error_table, error_table.get('error', {}).get('code', 500)

        except (ServerNotFoundError, TransportError) as err:
            return error_response(
                message='cannot connect to googleapis server to get spreadsheet values - ServerNotFoundError')

        return response_table, 200


    def raw_values_batch_get(self, spreadsheet_id, range_strings, pargs=None):
        pargs = {} if pargs is None else pargs

        try:
            response_table = self.sheets_service.spreadsheets().values().batchGet(
                spreadsheetId=spreadsheet_id,
                ranges=range_strings,
                majorDimension=pargs.get('majorDimension', 'ROWS'),
                valueRenderOption=pargs.get('valueRenderOption', 'FORMATTED_VALUE'),
                dateTimeRenderOption=pargs.get('dateTimeRenderOption', 'SERIAL_NUMBER')
            ).execute()

        except HttpError as err:
            error_table = json.loads(err.content)
            return error_table, error_table.get('error', {}).get('code', 500)

        except (ServerNotFoundError, TransportError) as err:
            return error_response(
                message='cannot connect to googleapis server to get values in requested spreadsheet - ServerNotFoundError')

        return response_table, 200


    ''' Some wrapper helper methods now on. which may help in some cases '''

    def sheets_dict_for(self, spreadsheet_id):
        '''
        just gives a dict mapping inner sheet ids to their titles in given spreadsheet.
        :method spreadsheet_details_for: gives much comprehensive result with all sheets list, and header fields. where as this one gives just small mapping from all gids to titles.

        :param spreadsheet_id: spreadsheetId
        :return: dict of sheet_gids to sheet titles in given spreadsheet. None if any error occurs
        '''
        sheets_response_table, code = self.spreadsheet_details_for(spreadsheet_id, pargs={"include_sheet_header_details":0})
        if 'error' in sheets_response_table:
            return None

        sheets_dict = {}
        for sheet in sheets_response_table['sheets']:
            sheets_dict[sheet['sheetGId']] = sheet['sheetTitle']

        return sheets_dict

    def to_csv(self, spreadsheet_id, sheet_id, csv_file, id_is_title=False, comment_out_first_row=True):
        sheets_dict = self.sheets_dict_for(spreadsheet_id)
        sheet_in_spreadsheet = sheet_id in list(sheets_dict.values()) if id_is_title else  sheet_id in list(sheets_dict.keys())
        if not sheet_in_spreadsheet:
            return
        sheet_title = sheet_id if id_is_title else sheets_dict[sheet_id]
        range = sheet_title

        sheet_values_table, code = self.raw_values_get(spreadsheet_id, range)
        if 'error' in sheet_values_table:
            return
        values = sheet_values_table['values']

        csv_writer = csv.writer(csv_file)
        gothdr = False
        for row in values:
            if not gothdr:
                gothdr = True
                row[0] = "#"+row[0] if comment_out_first_row else row[0]

            csv_writer.writerow(row)



def spreadsheet_id_from_url(url):
    found = re.search(r'/spreadsheets/d/([a-zA-Z0-9-_]+)', url)
    if not found:
        return None
    else:
        return found.groups()[0]

def spreadsheet_url_from_id(id):
    return 'https://docs.google.com/spreadsheets/d/{id}/edit'.format(id=id)






import json

import googleapiclient.discovery
from googleapiclient.errors import HttpError
from httplib2 import ServerNotFoundError
from google.auth.exceptions import TransportError

from ..oauth import creds_helper

SPREADSHEET_MIMETYPES = ['application/vnd.google-apps.spreadsheet']


def build_service(credentials):
    drive_service = googleapiclient.discovery.build('drive', 'v3', credentials=credentials, cache_discovery=False)
    return drive_service

def build_service_from_creds_file(creds_file, scopes, auth_through_service_account=False):
    credentials = creds_helper.credentials_from_file(creds_file, scopes, auth_through_service_account)
    return build_service(credentials)

def build_service_from_creds_string(creds_string, scopes, auth_through_service_account=False):
    credentials = creds_helper.credentials_from_string(creds_string, scopes, auth_through_service_account)
    return build_service(credentials)


def error_response(**kwargs):
    #print ('in error_response')
    response = {'error' : {}}
    is_error_inherited = 'inherited_error_table' in kwargs
    response_code = kwargs.get('code', kwargs['inherited_error_table']['error'].get('code', 500) if is_error_inherited else 500)
    response_message = kwargs.get('message', None)

    response['error']['code'] = response_code
    if response_message is not None :
        response['error']['message'] = response_message
    if is_error_inherited :
        response['error']['inherited_error'] = kwargs['inherited_error_table']['error']

    return (response), response_code

class GDrive(object):

    def __init__(self, drive_service, **kwargs):
        if not isinstance(drive_service, googleapiclient.discovery.Resource):
            raise TypeError('given service parameter is not googleapiclient.discovery.Resource object')

        self.drive_service = drive_service
        self.EXPOSE_BACKEND_ERRORS = kwargs.get('expose_backend_errors', True)


    @classmethod
    def from_credentials(cls, credentials, **kwargs):
        service = build_service(credentials)
        return cls(service, **kwargs)

    @classmethod
    def from_creds_file(cls, creds_file, scopes, auth_through_service_account=False, **kwargs):
        credentials = creds_helper.credentials_from_file(creds_file, scopes, auth_through_service_account=auth_through_service_account)
        return cls.from_credentials(credentials, **kwargs)

    @classmethod
    def from_creds_string(cls, creds_string, scopes, auth_through_service_account=False, **kwargs):
        credentials = creds_helper.credentials_from_string(creds_string, scopes, auth_through_service_account=auth_through_service_account)
        return cls.from_credentials(credentials, **kwargs)

    def list_of_files(self, pargs=None):
        '''
        this is generic function to retrieve list of files in all pages. you have to provide all query parameters like q string, orderBy etc.
        Note: see listOfFilesInFolder() method for listing files in a folder of desired type.

        :param pargs: query string parameters for google drive api 'list' method.
         see https://developers.google.com/drive/api/v3/reference/files/list for all avialable.
        :return: a tuple of (dictionary with 'files' key, and equivalent response http code).
        '''

        pargs = {} if pargs is None else pargs

        files_list = []
        page_token = None

        while True:
            try:
                present_page_response_table = self.drive_service.files().list(
                    pageToken=page_token,
                    **pargs
                ).execute()
            except HttpError as err:
                return error_response(message='error in retrieving list of files from gdrive directory',
                                      inherited_error_table=json.loads(err.content))
            except (ServerNotFoundError, TransportError) as err:
                return error_response(
                    message='cannot connect to googleapis server to get list of files from gdrive directory - ServerNotFoundError' if self.EXPOSE_BACKEND_ERRORS else 'cannot retrieve list of files from gdrive directory')

            present_page_files_list = present_page_response_table.get('files', [])

            files_list.extend(present_page_files_list)
            page_token = present_page_response_table.get('nextPageToken', None)
            if page_token is None:
                break

        our_response_table = {'kind': 'listOfFiles', 'files': files_list}
        return (our_response_table), 200

    def list_of_files_in_folder(self, folder_id, mime_types=None,
                                additional_pargs=None):
        '''
        returns an array of file objects, which have :param folderId: in their 'parent' attribute in google drive. note that, in gdrive, there is no concept of folders.
        it is just relations between differant files, like parent-child relation ships. a folder is also a file with proprietery mimeType. children files just have this folder-file id in their parent attribute.

        :param folder_id: id of the google drive folder
        :param mime_types: required mimeTypes in that folder.
        :param additional_pargs: any additional pargs, like orderBy.
        :return: a tuple of (dictionary with 'files' key, and equivalent http status code)
        '''
        # TODO should also accept custom mimeType expression including negations and all.

        additional_pargs = {} if additional_pargs is None else additional_pargs
        #mime_types = mime_types if mime_types else SPREADSHEET_MIMETYPES

        mime_types_and_clause = (' and ' + '(' + (' or '.join(["mimeType='{}'".format(mt) for mt in mime_types])) + ')') if mime_types else ''

        q_parameter = "'" + folder_id + "'" + ' in parents' + mime_types_and_clause
        order_by_parameter = additional_pargs.get('orderBy', 'name_natural')
        fields_parameter = 'nextPageToken, files(id, name, mimeType, createdTime, modifiedTime)'

        pargs = {
            'orderBy': order_by_parameter,
            'q': q_parameter,
            'fields': fields_parameter
        }

        our_response_table, code = self.list_of_files(pargs=pargs)

        if 'error' in our_response_table:
            return (our_response_table), code

        our_response_table['kind'] = 'listOfFilesInFolder'
        our_response_table['folderId'] = folder_id

        return (our_response_table), code





from .oauth import creds_helper
from .gsheets import helper as gsheets_helper
from .gdrive import helper as gdrive_helper


class GServices(object):

    def __init__(self, credentials):
        '''
        initializes gservices with credentials
        :param credentials: credentials
        '''
        self.__services = {}
        self.__credentials = credentials

    @classmethod
    def from_creds_file(cls, creds_path, scopes, auth_through_service_account=False):
        '''
        :param creds_path: path to credentials file
        :param scopes: scopes to be enabled
        :param auth_through_service_account: are credentials are of service_account?
        '''
        credentials = creds_helper.credentials_from_file(creds_path, scopes, auth_through_service_account=auth_through_service_account)
        return cls(credentials)

    @classmethod
    def from_creds_string(cls, creds_string, scopes, auth_through_service_account=False):
        credentials = creds_helper.credentials_from_string(creds_string, scopes, auth_through_service_account=auth_through_service_account)
        return cls(credentials)

    def __init_gdrive(self, force=False, **kwargs):
        if 'drive' in self.__services and self.__services.get('drive', None) is not None:
            if not force:
                return

        try:
            gdrive_discovery_service = gdrive_helper.build_service(self.__credentials)
        except Exception as e:
            raise e
        if gdrive_discovery_service is None:
            raise RuntimeError('cannot instantiate gdrive_discovery_service')
        self.__services['drive'] = gdrive_helper.GDrive(gdrive_discovery_service, **kwargs)

    def __init_gsheets(self, force=False, enable_drive_service_linking=False, **kwargs):
        if 'sheets' in self.__services and self.__services.get('sheets', None) is not None:
            if not force:
                return

        if enable_drive_service_linking:
            self.__init_gdrive(force=False, **kwargs)

        try:
            gsheets_discovery_service = gsheets_helper.build_service(self.__credentials)
        except Exception as e:
            raise e
        if gsheets_discovery_service is None:
            raise RuntimeError('cannot instantiate gsheets_discovery_service')
        self.__services['sheets'] = gsheets_helper.GSheets(gsheets_discovery_service, drive_service=self.__services.get("drive", None), **kwargs) if enable_drive_service_linking else gsheets_helper.GSheets(gsheets_discovery_service, **kwargs)


    def gdrive(self, refresh=False, **kwargs):
        self.__init_gdrive(force=refresh, **kwargs)
        return self.__services['drive']


    def gsheets(self, refresh=False, enable_drive_service_linking=False, **kwargs):
        self.__init_gsheets(force=refresh, enable_drive_service_linking=enable_drive_service_linking, **kwargs)
        return self.__services['sheets']



# -*- coding: utf-8 -*-

import json

import google.oauth2.credentials
from google.oauth2 import service_account



def credentials_from_file(creds_file, scopes, auth_through_service_account=False):
    credentials = None;

    try:
        if auth_through_service_account:
            credentials = service_account.Credentials.from_service_account_info(json.loads(open(creds_file, 'rb').read().decode('utf-8')), scopes=scopes)
        else:
            print(creds_file)
            credentials = google.oauth2.credentials.Credentials(**json.loads(open(creds_file, 'rb').read().decode('utf-8'))) #json.load() producing UnicodeDecodeError in production wsgi env some times.

    except IOError as err:
        raise IOError('error in loading credentials file from given path');

    return credentials


def credentials_from_string(creds_string, scopes, auth_through_service_account=False):
    credentials = None;

    if auth_through_service_account:
        credentials = service_account.Credentials.from_service_account_info(json.loads(creds_string, encoding='utf-8'),scopes=scopes)
    else:
        credentials = google.oauth2.credentials.Credentials(**json.loads(creds_string, encoding='utf-8'))

    return credentials


def credentials_from_dict(creds_dict, scopes, auth_through_service_account=False):
    credentials = None;

    if auth_through_service_account:
        credentials = service_account.Credentials.from_service_account_info(creds_dict, scopes=scopes)
    else:
        credentials = google.oauth2.credentials.Credentials(**creds_dict)
    return credentials


def credentials_to_dict(credentials):
    cdict = {'token': credentials.token,
                'refresh_token': credentials.refresh_token,
                'token_uri': credentials.token_uri,
                'client_id': credentials.client_id,
                'client_secret': credentials.client_secret,
                'scopes': credentials.scopes}
    return cdict







from setuptools import setup

try:
    with open("README.md", "r") as fh:
        long_description = fh.read()
except:
    long_description = ''

setup(
    name='google-services-helper',
    version='1.0.3',
    packages=['vedavaapi', 'vedavaapi.google_helper', 'vedavaapi.google_helper.oauth', 'vedavaapi.google_helper.gdrive',
              'vedavaapi.google_helper.gsheets'],
    url='https://github.com/vedavaapi',
    author='vedavaapi',
    author_email='vvsvmb151@gmail.com',
    description='google api helper for vedavaapi project',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['flask', 'google-auth', 'google-auth-oauthlib', 'google-auth-httplib2', 'google-api-python-client'],
    classifiers=(
            "Programming Language :: Python :: 2.7",
            "Operating System :: OS Independent",
    )
)
